@@ -139,7 +139,7 @@ struct Impl_Feeds {
 
 static iFeeds feeds_;
 
-#define maxConcurrentRequests_Feeds 4
+#define maxConcurrentRequests_Feeds 10
 
 static iBool isInitialized_Feeds_(const iFeeds *d) {
     return d->mtx != NULL;
@@ -753,6 +753,15 @@ void deinit_Feeds(void) {
     deinit_SortedArray(&d->entries);
 }
 
+void resetKnownEntries_Feeds(void) {
+    iFeeds *d = &feeds_;
+    iString *saveDir = copy_String(&d->saveDir);
+    deinit_Feeds();
+    remove(cstrCollect_String(concatCStr_Path(saveDir, feedsFilename_Feeds_)));
+    init_Feeds(cstr_String(saveDir));
+    delete_String(saveDir);
+}
+
 void refresh_Feeds(void) {
     startWorker_Feeds_(&feeds_);
 }
@@ -862,6 +871,20 @@ const iPtrArray *listEntries_Feeds(void) {
     return list;
 }
 
+const iPtrArray *listMatchingEntries_Feeds(const char *urlPrefix) {
+    iFeeds *d = &feeds_;
+    lock_Mutex(d->mtx);
+    iPtrArray *list = collectNew_PtrArray();
+    iForEach(PtrArray, i, &d->entries.values) {
+        iFeedEntry *entry = i.ptr;
+        if (startsWith_String(&entry->url, urlPrefix)) {
+            pushBack_PtrArray(list, entry);
+        }
+    }
+    unlock_Mutex(d->mtx);
+    return list;
+}
+
 size_t numSubscribed_Feeds(void) {
     return size_PtrArray(listSubscriptions_());
 }
