@@ -21,6 +21,7 @@ ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 
 #include "prefs.h"
+#include "ui/sidebarwidget.h"
 #include "app.h"
 
 #include <assert.h>
@@ -40,8 +41,9 @@ void init_Prefs(iPrefs *d) {
     d->translationIgnorePre         = iTrue;
     d->recentMenuBarIndex           = 0;
     d->useSystemTheme               = iTrue;
-    d->systemPreferredColorTheme[0] = d->systemPreferredColorTheme[1] = -1;
-    d->theme                                                          = dark_ColorTheme;
+    d->systemPreferredColorTheme[0] = pureBlack_ColorTheme;
+    d->systemPreferredColorTheme[1] = pureWhite_ColorTheme;
+    d->theme                        = dark_ColorTheme;
     d->accent                   = isAppleDesktop_Platform() ? system_ColorAccent : cyan_ColorAccent;
     d->customFrame              = iFalse; /* needs some more work to be default */
     d->retainWindowSize         = iTrue;
@@ -55,15 +57,35 @@ void init_Prefs(iPrefs *d) {
     d->navbarActions[1]         = forward_ToolbarAction;
     d->navbarActions[2]         = leftSidebar_ToolbarAction;
     d->navbarActions[3]         = home_ToolbarAction;
-#if defined(iPlatformAndroidMobile)
+#if defined (iPlatformAndroidMobile)
     /* Android has a system-wide back button so no need to have a duplicate. */
     d->toolbarActions[0] = closeTab_ToolbarAction;
 #else
     d->toolbarActions[0] = back_ToolbarAction;
 #endif
-    d->toolbarActions[1]   = forward_ToolbarAction;
+    d->toolbarActions[1] = forward_ToolbarAction;
+    iZap(d->sidebarModeEnabled);
+    if (deviceType_App() == phone_AppDeviceType) {
+        /* Phone layout has only the one sidebar. */
+        iForIndices(i, d->sidebarModeEnabled[0]) {
+            d->sidebarModeEnabled[0][i] = (i != subscriptions_SidebarMode &&
+                                           i != identities_SidebarMode &&
+                                           i != openDocuments_SidebarMode);
+        }
+    }
+    else {
+        d->sidebarModeEnabled[0][bookmarks_SidebarMode] = iTrue;
+        d->sidebarModeEnabled[0][feedEntries_SidebarMode] = iTrue;
+        d->sidebarModeEnabled[0][subscriptions_SidebarMode] = iTrue;
+        d->sidebarModeEnabled[0][identities_SidebarMode] = iTrue;
+        d->sidebarModeEnabled[1][documentOutline_SidebarMode] = iTrue;
+        d->sidebarModeEnabled[1][siteStructure_SidebarMode] = iTrue;
+        d->sidebarModeEnabled[1][openDocuments_SidebarMode] = iTrue;
+        d->sidebarModeEnabled[1][history_SidebarMode] = iTrue;
+    }
     d->sideIcon            = iTrue;
     d->hideToolbarOnScroll = iTrue;
+    d->hideTabBar          = iFalse;
     d->blinkingCursor      = iTrue;
     if (deviceType_App() == phone_AppDeviceType) {
         d->bottomNavBar = iTrue;
@@ -82,6 +104,7 @@ void init_Prefs(iPrefs *d) {
     d->detachedPrefs                          = iTrue;
     d->pinSplit                               = 1;
     d->feedInterval                           = fourHours_FeedInterval;
+    d->italicQuote                            = iTrue;
     d->time24h                                = iTrue;
     d->returnKey                              = default_ReturnKeyBehavior;
     d->retainTabs                             = iTrue;
