@@ -22,6 +22,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 
 #pragma once
 
+#include "../gmdocument.h"
 #include "widget.h"
 #include <the_Foundation/stream.h>
 
@@ -29,32 +30,58 @@ iDeclareType(GmDocument)
 iDeclareType(GmIdentity)
 iDeclareType(GmRequest)
 iDeclareType(History)
+iDeclareType(Banner)
+iDeclareType(ScrollWidget)
 
 iDeclareWidgetClass(DocumentWidget)
 iDeclareObjectConstruction(DocumentWidget)
 
-void    cancelAllRequests_DocumentWidget(iDocumentWidget *);
+iDocumentWidget *   duplicate_DocumentWidget        (const iDocumentWidget *);
+void                cancelAllRequests_DocumentWidget(iDocumentWidget *);
 
-void    serializeState_DocumentWidget   (const iDocumentWidget *, iStream *outs);
+void    serializeState_DocumentWidget   (const iDocumentWidget *, iStream *outs, iBool withContent);
 void    deserializeState_DocumentWidget (iDocumentWidget *, iStream *ins);
 
-iDocumentWidget *   duplicate_DocumentWidget        (const iDocumentWidget *);
 iHistory *          history_DocumentWidget          (iDocumentWidget *);
-
+iWidget *           footerButtons_DocumentWidget    (const iDocumentWidget *);
+iScrollWidget *     scrollBar_DocumentWidget        (const iDocumentWidget *);
 const iString *     url_DocumentWidget              (const iDocumentWidget *);
 const iBlock *      sourceContent_DocumentWidget    (const iDocumentWidget *);
+iTime               sourceTime_DocumentWidget       (const iDocumentWidget *);
 const iGmDocument * document_DocumentWidget         (const iDocumentWidget *);
 const iString *     bookmarkTitle_DocumentWidget    (const iDocumentWidget *);
 const iString *     feedTitle_DocumentWidget        (const iDocumentWidget *);
-int                 documentWidth_DocumentWidget    (const iDocumentWidget *);
-const iGmIdentity * identity_DocumentWidget         (const iDocumentWidget *);
-int                 generation_DocumentWidget       (const iDocumentWidget *);
+uint32_t            findBookmarkId_DocumentWidget   (const iDocumentWidget *);
 
-iBool               isRequestOngoing_DocumentWidget (const iDocumentWidget *);
-iBool               isSourceTextView_DocumentWidget (const iDocumentWidget *);
-iBool               isIdentityPinned_DocumentWidget    (const iDocumentWidget *);
+int                 documentWidth_DocumentWidget        (const iDocumentWidget *);
+int                 footerHeight_DocumentWidget         (const iDocumentWidget *);
+int                 phoneToolbarHeight_DocumentWidget   (const iDocumentWidget *);
+int                 phoneBottomNavbarHeight_DocumentWidget(const iDocumentWidget *);
+iRangecc            selectionMark_DocumentWidget        (const iDocumentWidget *);
+const iGmIdentity * identity_DocumentWidget             (const iDocumentWidget *);
+int                 generation_DocumentWidget           (const iDocumentWidget *);
+iBool               isRequestOngoing_DocumentWidget     (const iDocumentWidget *);
+iBool               isPrerenderingAllowed_DocumentWidget(const iDocumentWidget *);
+iBool               isSourceTextView_DocumentWidget     (const iDocumentWidget *);
+iBool               isIdentityPinned_DocumentWidget     (const iDocumentWidget *);
 iBool               isSetIdentityRetained_DocumentWidget(const iDocumentWidget *, const iString *dstUrl);
-iBool               isAutoReloading_DocumentWidget  (const iDocumentWidget *);
+iBool               isAutoReloading_DocumentWidget      (const iDocumentWidget *);
+iBool               isHoverAllowed_DocumentWidget       (const iDocumentWidget *);
+iBool               noHoverWhileScrolling_DocumentWidget(const iDocumentWidget *);
+iBool               isShowingLinkNumbers_DocumentWidget (const iDocumentWidget *);
+iBool               isBlank_DocumentWidget              (const iDocumentWidget *);
+iBool               isUnseen_DocumentWidget             (const iDocumentWidget *);
+iMediaRequest *     findMediaRequest_DocumentWidget     (const iDocumentWidget *, iGmLinkId linkId);
+
+size_t              ordinalBase_DocumentWidget          (const iDocumentWidget *);
+iChar               linkOrdinalChar_DocumentWidget      (const iDocumentWidget *, size_t ord);
+
+enum iWheelSwipeState {
+    none_WheelSwipeState,
+    direct_WheelSwipeState,
+};
+
+enum iWheelSwipeState   wheelSwipeState_DocumentWidget  (const iDocumentWidget *);
 
 enum iDocumentWidgetSetUrlFlags {
     useCachedContentIfAvailable_DocumentWidgetSetUrlFlag = iBit(1),
@@ -76,4 +103,12 @@ void    setSource_DocumentWidget        (iDocumentWidget *, const iString *sourc
 
 void    takeRequest_DocumentWidget      (iDocumentWidget *, iGmRequest *finishedRequest); /* ownership given */
 
-void    updateSize_DocumentWidget       (iDocumentWidget *);
+void    documentRunsInvalidated_DocumentWidget  (iDocumentWidget *);
+void    updateSize_DocumentWidget               (iDocumentWidget *);
+void    updateHoverLinkInfo_DocumentWidget      (iDocumentWidget *, uint16_t linkId);
+void    scrollBegan_DocumentWidget              (iAnyObject *, int, uint32_t); /* SmoothScroll callback */
+void    aboutToScrollView_DocumentWidget        (iDocumentWidget *, int scrollMax);
+void    didScrollView_DocumentWidget            (iDocumentWidget *);
+
+void    animate_DocumentWidget                  (iAny *); /* ticker */
+void    refreshWhileScrolling_DocumentWidget    (iAny *); /* ticker */
