@@ -30,7 +30,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 #endif
 
 #if defined (LAGRANGE_ENABLE_FRIBIDI)
-#   include <fribidi/fribidi.h>
+#   include <fribidi.h>
 #endif
 
 iColor fgColor_AttributedRun(const iAttributedRun *d) {
@@ -62,7 +62,7 @@ iColor fgColor_AttributedRun(const iAttributedRun *d) {
                 fg = rgb_HSLColor(fgHsl);
             }
             return fg;
-        }        
+        }
         return d->fgColor_;
     }
     if (d->attrib.fgColorId == none_ColorId) {
@@ -246,61 +246,89 @@ static void prepare_AttributedText_(iAttributedText *d, int overrideBaseDir, iCh
             init_RegExpMatch(&m);
             if (match_RegExp(current_Text()->ansiEscape, srcPos, d->source.end - srcPos, &m)) {
                 finishRun_AttributedText_(d, &run, pos - 1);
-                const int ansi = current_Text()->ansiFlags;
-                if (ansi && capturedRange_RegExpMatch(&m, 2).start[0] ==
-                                'm' /* Select Graphic Rendition */) {
-                    const iRangecc sequence = capturedRange_RegExpMatch(&m, 1);
-                    /* Note: This styling is hardcoded to match `typesetOneLine_RunTypesetter_()`. */
-                    if (ansi & allowFontStyle_AnsiFlag && equal_Rangecc(sequence, "1")) {
-                        run.attrib.bold = iTrue;
-                        run.attrib.regular = iFalse;
-                        run.attrib.light = iFalse;
-                        if (d->baseFgColorId == tmParagraph_ColorId) {
-                            setFgColor_AttributedRun_(&run, tmFirstParagraph_ColorId);
+                const int ansi = current_Text()->ansiFlags; /* styling enabled */
+                const char mode = capturedRange_RegExpMatch(&m, 2).start[0];
+                const iRangecc sequence = capturedRange_RegExpMatch(&m, 1);
+                if (ansi && mode == 'm' /* Select Graphic Rendition */) {
+                    for (const char *seqPos = sequence.start; seqPos < sequence.end; ) {
+                        /* One sequence may have multiple codes. */
+                        char *argEnd;
+                        const int arg = strtoul(seqPos, &argEnd, 10);
+                        /* Note: This styling is hardcoded to match `typesetOneLine_RunTypesetter_()`. */
+                        if (arg == 1) {
+                            if (ansi & allowFontStyle_AnsiFlag) {
+                                run.attrib.bold = iTrue;
+                                run.attrib.regular = iFalse;
+                                run.attrib.light = iFalse;
+                                if (d->baseFgColorId == tmParagraph_ColorId) {
+                                    setFgColor_AttributedRun_(&run, tmFirstParagraph_ColorId);
+                                }
+                                attribFont = font_Text(fontWithStyle_Text(fontId_Text(d->baseFont),
+                                                                           bold_FontStyle));
+                            }
+                        }
+                        else if (arg == 2) {
+                            if (ansi & allowFontStyle_AnsiFlag) {
+                                run.attrib.light = iTrue;
+                                run.attrib.regular = iFalse;
+                                run.attrib.bold = iFalse;
+                                attribFont = font_Text(fontWithStyle_Text(fontId_Text(d->baseFont),
+                                                                           light_FontStyle));
+                            }
+                        }
+                        else if (arg == 3) {
+                            if (ansi & allowFontStyle_AnsiFlag) {
+                                run.attrib.italic = iTrue;
+                                attribFont = font_Text(fontWithStyle_Text(fontId_Text(d->baseFont),
+                                                                           italic_FontStyle));
+                            }
+                        }
+                        else if (arg == 10) {
+                            if (ansi & allowFontStyle_AnsiFlag) {
+                                run.attrib.regular = iTrue;
+                                run.attrib.bold = iFalse;
+                                run.attrib.light = iFalse;
+                                run.attrib.italic = iFalse;
+                                attribFont = font_Text(fontWithStyle_Text(fontId_Text(d->baseFont),
+                                                                           regular_FontStyle));
+                            }
+                        }
+                        else if (arg == 11) {
+                            if (ansi & allowFontStyle_AnsiFlag) {
+                                run.attrib.monospace = iTrue;
+                                setFgColor_AttributedRun_(&run, tmPreformatted_ColorId);
+                                attribFont = font_Text(fontWithFamily_Text(fontId_Text(d->baseFont),
+                                                                            monospace_FontId));
+                            }
+                        }
+                        else if (arg == 0) {
+                            run.attrib.regular = iFalse;
+                            run.attrib.bold = iFalse;
+                            run.attrib.light = iFalse;
+                            run.attrib.italic = iFalse;
+                            run.attrib.monospace = iFalse;
+                            attribFont = run.font = d->baseFont;
+                            setFgColor_AttributedRun_(&run, d->baseFgColorId);
+                            setBgColor_AttributedRun_(&run, none_ColorId);
+                        }
+                        else {
+                            const char *end;
+                            ansiColors_Color((iRangecc){ seqPos, sequence.end },
+                                             d->baseFgColorId,
+                                             none_ColorId,
+                                             run.attrib.bold != 0,
+                                             ansi & allowFg_AnsiFlag ? &run.fgColor_ : NULL,
+                                             ansi & allowBg_AnsiFlag ? &run.bgColor_ : NULL,
+                                             &end);
+                            argEnd = (char *) end;
+                        }
+                        seqPos = argEnd;
+                        if (seqPos < sequence.end) {
+                            if (*seqPos == ';') {
+                                seqPos++;
+                            }
+                            else break; /* malformed or didn't understand */
                         }
-                        attribFont = font_Text(fontWithStyle_Text(fontId_Text(d->baseFont),
-                                                                   bold_FontStyle));
-                    }
-                    else if (ansi & allowFontStyle_AnsiFlag && equal_Rangecc(sequence, "2")) {
-                        run.attrib.light = iTrue;
-                        run.attrib.regular = iFalse;
-                        run.attrib.bold = iFalse;
-                        attribFont = font_Text(fontWithStyle_Text(fontId_Text(d->baseFont),
-                                                                   light_FontStyle));
-                    }
-                    else if (ansi & allowFontStyle_AnsiFlag && equal_Rangecc(sequence, "3")) {
-                        run.attrib.italic = iTrue;
-                        attribFont = font_Text(fontWithStyle_Text(fontId_Text(d->baseFont),
-                                                                   italic_FontStyle));
-                    }
-                    else if (ansi & allowFontStyle_AnsiFlag && equal_Rangecc(sequence, "10")) {
-                        run.attrib.regular = iTrue;
-                        run.attrib.bold = iFalse;
-                        run.attrib.light = iFalse;
-                        run.attrib.italic = iFalse;
-                        attribFont = font_Text(fontWithStyle_Text(fontId_Text(d->baseFont),
-                                                                   regular_FontStyle));
-                    }
-                    else if (ansi & allowFontStyle_AnsiFlag && equal_Rangecc(sequence, "11")) {
-                        run.attrib.monospace = iTrue;
-                        setFgColor_AttributedRun_(&run, tmPreformatted_ColorId);
-                        attribFont = font_Text(fontWithFamily_Text(fontId_Text(d->baseFont),
-                                                                    monospace_FontId));
-                    }
-                    else if (equal_Rangecc(sequence, "0")) {
-                        run.attrib.regular = iFalse;
-                        run.attrib.bold = iFalse;
-                        run.attrib.light = iFalse;
-                        run.attrib.italic = iFalse;
-                        run.attrib.monospace = iFalse;
-                        attribFont = run.font = d->baseFont;
-                        setFgColor_AttributedRun_(&run, d->baseFgColorId);
-                        setBgColor_AttributedRun_(&run, none_ColorId);
-                    }
-                    else {
-                        ansiColors_Color(sequence, d->baseFgColorId, none_ColorId,
-                                         ansi & allowFg_AnsiFlag ? &run.fgColor_ : NULL,
-                                         ansi & allowBg_AnsiFlag ? &run.bgColor_ : NULL);
                     }
                 }
                 pos += length_Rangecc(capturedRange_RegExpMatch(&m, 0));
@@ -334,7 +362,7 @@ static void prepare_AttributedText_(iAttributedText *d, int overrideBaseDir, iCh
             finishRun_AttributedText_(d, &run, pos + 1);
             continue;
         }
-        if (isControl_Char(ch)) {
+        if (isControl_Char(ch) || ch == 0x202f /* NNBSP */) {
             continue;
         }
         iAssert(run.font != NULL);
@@ -401,13 +429,15 @@ static void prepare_AttributedText_(iAttributedText *d, int overrideBaseDir, iCh
     }
 #if 0
     const int *logToVis = constData_Array(&d->logicalToVisual);
-    printf("[AttributedText] %zu runs:\n", size_Array(&d->runs));
+    printf("[AttributedText] fnt:%d %zu runs:\n", fontId_Text(d->baseFont), size_Array(&d->runs));
     iConstForEach(Array, i, &d->runs) {
         const iAttributedRun *run = i.value;
-        printf("  %zu %s fnt:%d log:%d...%d vis:%d...%d {%s}\n",
+        printf("  %zu %s fnt:%d(%c:%s) log:%d...%d vis:%d...%d {%s}\n",
                index_ArrayConstIterator(&i),
                run->attrib.isRTL ? "<-" : "->",
-               fontId_Text_(run->font),
+               fontId_Text(run->font),
+               isMonospaced_Font(run->font) ? 'M' : 'v',
+               cstr_String(&run->font->spec->name),
                run->logical.start, run->logical.end - 1,
                logToVis[run->logical.start], logToVis[run->logical.end - 1],
                cstr_Rangecc(sourceRange_AttributedText_(d, run->logical)));
@@ -447,13 +477,13 @@ void deinit_AttributedText(iAttributedText *d) {
 
 iTextMetrics measure_WrapText(iWrapText *d, int fontId) {
     iTextMetrics tm;
-    tm.bounds = run_Font(font_Text(fontId),
-                         &(iRunArgs){ .mode        = measure_RunMode | runFlags_FontId(fontId),
-                                      .text        = d->text,
-                                      .wrap        = d,
-                                      .justify     = d->justify,
-                                      .layoutBound = d->justify ? d->maxWidth : 0,
-                                      .cursorAdvance_out = &tm.advance });
+    run_Font(font_Text(fontId),
+             &(iRunArgs){ .mode        = measure_RunMode | runFlags_FontId(fontId),
+                          .text        = d->text,
+                          .wrap        = d,
+                          .justify     = d->justify,
+                          .layoutBound = d->justify ? d->maxWidth : 0,
+                          .metrics_out = &tm });
     return tm;
 }
 
@@ -471,6 +501,9 @@ iTextMetrics draw_WrapText(iWrapText *d, int fontId, iInt2 pos, int color) {
         const int width = d->mode == word_WrapTextMode
                               ? tryAdvance_Text(fontId, text, d->maxWidth, &endPos).x
                               : tryAdvanceNoWrap_Text(fontId, text, d->maxWidth, &endPos).x;
+        if (endPos == text.start) {
+            break; /* too tight for even a single character */
+        }
         notify_WrapText(d, endPos, (iTextAttrib){ .fgColorId = color }, 0, width);
         drawRange_Text(fontId, pos, color, (iRangecc){ text.start, endPos });
         text.start = endPos;
@@ -480,18 +513,19 @@ iTextMetrics draw_WrapText(iWrapText *d, int fontId, iInt2 pos, int color) {
     }
     tm.advance = sub_I2(pos, orig);
 #else
-    tm.bounds = run_Font(font_Text(fontId),
-              &(iRunArgs){ .mode  = draw_RunMode | runFlags_FontId(fontId) |
-                                    (color & permanent_ColorId ? permanentColorFlag_RunMode : 0) |
-                                    (color & fillBackground_ColorId ? fillBackground_RunMode : 0),
-                           .text  = d->text,
-                           .pos   = pos,
-                           .wrap  = d,
-                           .justify = d->justify,
-                           .layoutBound = d->justify ? d->maxWidth : 0,
-                           .color = color & mask_ColorId,
-                           .cursorAdvance_out = &tm.advance,
-    });
+    run_Font(font_Text(fontId),
+             &(iRunArgs){
+                 .mode = draw_RunMode | runFlags_FontId(fontId) |
+                         (color & permanent_ColorId ? permanentColorFlag_RunMode : 0) |
+                         (color & fillBackground_ColorId ? fillBackground_RunMode : 0),
+                 .text = d->text,
+                 .pos = pos,
+                 .wrap = d,
+                 .justify = d->justify,
+                 .layoutBound = d->justify ? d->maxWidth : 0,
+                 .color = color & mask_ColorId,
+                 .metrics_out = &tm,
+             });
 #endif
     return tm;
 }
